<?php
	global $db, $messages, $fixes, $fix_id, $settings;

	require_once('content/classes/class.languages.php');

	if (!defined('DS')) define('DS',DIRECTORY_SEPARATOR);

	global $backup_suffix;
	$backup_suffix = '382r1-full';

	function getChmodMode($filepath, $refresh = false)
	{
		static $_stats = array();

		if (!isset($_stats[$filepath]) || $refresh)
		{
			if ($refresh) clearstatcache();

			$_stats[$filepath] = stat($filepath);
		}

		return sprintf('0%o', 0777 & $_stats[$filepath]['mode']);
	}

	function hotFixRemoveDirectory($dir) {
		global $messages;
		$files = glob($dir.'*', GLOB_MARK);

		foreach($files as $file){
			$info = pathinfo($file);

			if( is_dir($file))
				hotFixRemoveDirectory($file);
			else
			{
				@unlink($file) or $messages[] = array("icon"=>"ic_error.gif", "text"=>"<b>Please remove this file manually as it is not needed:</b> {$file}");
			}
		}
		if (is_dir($dir)) @rmdir($dir);
	}


	if (!function_exists('getDirectoryPermissionMode'))
	{
		/**
		 * Returns the set server directory permission mode
		 *
		 * @return int The directory permission mode
		 */
		function getDirectoryPermissionMode()
		{
			if (defined('DIRECTORY_PERMISSION_MODE')) return DIRECTORY_PERMISSION_MODE;

			global $settings;

			return octdec($settings['DirectoryPermissionMode']);
		}
	}


	if (!function_exists('getFilePermissionMode'))
	{
		/**
		 * Returns the set server file permission mode
		 *
		 * @return int The file permission mode
		 */
		function getFilePermissionMode()
		{
			if (defined('FILE_PERMISSION_MODE')) return FOLDER_PERMISSION_MODE;

			global $settings;

			return octdec($settings['FilePermissionMode']);
		}
	}

	try
	{
		$update_base_path = 'content/cache/update/'.$fix_id;

		if (file_exists($update_base_path))
		{

			// update errors
			$file_update_errors = array();

			$files = traverse($update_base_path);

			foreach ($files as $new_file)
			{
				$old_file = str_replace($update_base_path.'/', '', $new_file);

				$contents = file_get_contents($new_file);
				file_put_contents($old_file, $contents) or $update_errors[] = $old_file;
			}

			if (count($file_update_errors) > 0)
			{
				$message = '<p style="font-weight: bold;">The following files could not be updated:</p>
			<ul style="font-family: Monaco, Courier;list-style: none; padding: 0;">';
				foreach ($file_update_errors as $filename)
				{
					$message .= '<li style="margin: 10px;">'.$filename.'</li>';
				}
				$message .= "</ul>";

				throw new Exception($message);
			}
			else
			{
				hotFixRemoveDirectory($update_base_path);
			}
		}

	} catch (Exception $e) {

		print '
		<style type="text/css">
			.update-error { }
			.update-error-header { }
			.update-error-contact { border: 1px solid #cccccc; padding: 20px; }
			.update-error-message { border: 1px solid #cccccc; padding: 20px; background-color: #ffc; }
		</style>
		<div class="update-error">
			<h2 class="update-error-header">We\'re sorry, the hotfix updater has encountered the following issues:</h2>
			<div class="update-error-message">'.$e->getMessage().'</div>
			<p class="update-error-contact">Please contact technical support for assistance regarding the '.$fixes[$fix_id]['name'].' upgrade.</p>
		</div>
	';

		// unlink the installed-xml file so it appears in their admin area
		@unlink("content/cache/update/installed-".$fixes[$fix_id]['base_name']);

		exit;
	}

	

if (!isset($settings['UpdateStep']))
{
	$db->query('INSERT INTO '.DB_PREFIX.'settings (name, value) VALUES("UpdateStep", "0")');
	$updateStep = 0;
}
else
{
	$updateStep = intval($settings['UpdateStep']);
}

if (isset($_REQUEST['startover']))
{
	$updateStep = 0;
}
else if ($updateStep > 0 && !isset($_REQUEST['continue']))
{
	echo
		'We detected that update script was previously ran. Please choose what you would like to do next:<br>'.
		'<a href="update-to-380-r2-full.php?startover=1">Start Over</a> - OR - <a href="update-to-380-r2-full.php?continue=1">Continue</a>';
}

/**
 * Database changes
 */
if ($updateStep < 1)
{
	$db->query("START TRANSACTION");
	$db->query("ALTER TABLE ".DB_PREFIX."admins CHANGE COLUMN rights rights set('all','demo','categories','products','users','orders','emails','admins','global','business','content','database','stats','marketing','ccs','recurring','support') NOT NULL DEFAULT '';");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='1' WHERE name='UpdateStep'");
	$db->query("COMMIT");

}

if ($updateStep < 2)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("emails");
	if (!in_array('emails.receives_marketing', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."emails ADD COLUMN receives_marketing enum('Yes','No') NOT NULL DEFAULT 'No';");
	$db->query('UPDATE '.DB_PREFIX.'emails SET receives_marketing="Yes" WHERE newsletters="Yes"');
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='2' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 3)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("endicia_shippinglabels");
	if (!in_array('endicia_shippinglabels.date_created', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."endicia_shippinglabels ADD COLUMN date_created datetime NOT NULL DEFAULT '0000-00-00 00:00:00';");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='3' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 4)
{
	$db->query("START TRANSACTION");
	$db->query('UPDATE '.DB_PREFIX.'endicia_shippinglabels sl INNER JOIN '.DB_PREFIX.'orders o ON sl.order_id = o.oid SET date_created = o.status_date');
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='4' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 5)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("global_attributes");
	if (!in_array('global_attributes.track_inventory', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."global_attributes ADD COLUMN track_inventory tinyint(1) unsigned NOT NULL DEFAULT '0';");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='5' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 6)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("manufacturers");
	if (!in_array('manufacturers.image', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."manufacturers ADD COLUMN image varchar(250) DEFAULT NULL;");
	if (!in_array('manufacturers.meta_title', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."manufacturers ADD COLUMN meta_title text;");
	if (!in_array('manufacturers.meta_description', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."manufacturers ADD COLUMN meta_description text;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='6' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 7)
{
	$db->query("START TRANSACTION");
	$manufacturersResult = $db->query('SELECT manufacturer_id AS id FROM '.DB_PREFIX.'manufacturers');
	while ($item = $db->moveNext($manufacturersResult, MYSQL_ASSOC))
	{
		$imageFile = 'images/manufacturers/'.$item['id'].'.';
		$imageExt = false;
		if (is_file($imageFile.'jpg')) $imageExt = 'jpg';
		else if (is_file($imageFile.'png')) $imageExt = 'png';
		else if (is_file($imageFile.'gif')) $imageExt = 'gif';

		if ($imageExt) $db->query('UPDATE '.DB_PREFIX.'manufacturers SET image="'.$db->escape($imageFile.$imageExt).'" WHERE manufacturer_id='.intval($item['id']));
	}
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='7' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 8)
{
	$db->query("START TRANSACTION");

	$columnCheck = columnCheck("orders");
	if (!in_array('orders.fulfillment_status', $columnCheck))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders ADD COLUMN fulfillment_status enum('pending','partial','completed') NOT NULL DEFAULT 'pending' AFTER payment_status;");
		$db->query("UPDATE ".DB_PREFIX."orders SET fulfillment_status='completed' WHERE status='Completed' AND payment_status='Received'");
	}
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='8' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 9)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("orders_content");
	if (!in_array('orders_content.fulfillment_status', $columnCheck))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN fulfillment_status enum('pending','partial','completed') NOT NULL DEFAULT 'pending' AFTER is_stock_changed;");
		$db->query("UPDATE ".DB_PREFIX."orders_content oc INNER JOIN  ".DB_PREFIX."orders o ON oc.oid=o.oid AND o.status='Completed' AND o.payment_status='Received' SET oc.fulfillment_status='completed'");
	}
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='9' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 10)
{
	$db->query("START TRANSACTION");
	if (!in_array('orders_content.fulfilled_quantity', $columnCheck))
	{
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN fulfilled_quantity int(10) unsigned NOT NULL DEFAULT '0' AFTER fulfillment_status;");
		$db->query("UPDATE ".DB_PREFIX."orders_content oc INNER JOIN  ".DB_PREFIX."orders o ON oc.oid=o.oid AND o.status='Completed' AND o.payment_status='Received' SET oc.fulfilled_quantity=oc.admin_quantity");
	}
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='10' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 11)
{
	$db->query("START TRANSACTION");
	$db->query("ALTER TABLE ".DB_PREFIX."payment_methods CHANGE COLUMN active active enum('Yes','No','Disabled') NOT NULL DEFAULT 'No';");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='11' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 12)
{
	$db->query("START TRANSACTION");
	$db->query("ALTER TABLE ".DB_PREFIX."products CHANGE COLUMN price2 price2 decimal(20,5) unsigned DEFAULT NULL;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='12' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 13)
{
	$db->query("START TRANSACTION");
	$db->query("UPDATE ".DB_PREFIX."products SET price2=NULL WHERE price2=0");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='13' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 14)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("products_gifts");
	if (!in_array('products_gifts.product_gift_id', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."products_gifts ADD COLUMN product_gift_id int(10) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY FIRST;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='14' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}


if ($updateStep < 15)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("products_inventory");
	if (!in_array('products_inventory.product_sku', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."products_inventory ADD COLUMN product_sku varchar(64) NOT NULL DEFAULT '' AFTER is_active;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='15' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 16)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("products_reviews");
	if (!in_array('products_reviews.status', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."products_reviews ADD COLUMN status enum('pending','approved','declined') NOT NULL DEFAULT 'pending' AFTER rating;");
	$db->query("UPDATE ".DB_PREFIX."products_reviews SET status = 'approved' WHERE is_visible = 1 AND approved = 1;");
	//$db->query("UPDATE ".DB_PREFIX."products_reviews SET status = 'declined' WHERE is_visible = 0 AND approved = 0;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='16' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 17)
{
	$db->query("START TRANSACTION");
	$columnCheck = columnCheck("users");
	if (!in_array('users.receives_marketing', $columnCheck)) $db->query("ALTER TABLE ".DB_PREFIX."users ADD COLUMN receives_marketing enum('Yes', 'No') NOT NULL default 'No' AFTER removed;");
	$db->query('UPDATE '.DB_PREFIX.'users SET receives_marketing="Yes" WHERE newsletters="Yes"');
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='17' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

/**
 * Applications
 */
if ($updateStep < 18)
{
	$db->query("START TRANSACTION");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."enabled_apps (id int(11) unsigned NOT NULL AUTO_INCREMENT, `key` varchar(64) NOT NULL, primary key(id)) ENGINE=MyISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='18' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 19)
{
	$db->query("START TRANSACTION");
	$counts = $db->selectOne('SELECT COUNT(*) as c FROM '.DB_PREFIX.'products WHERE google_product_type != ""', MYSQL_ASSOC);
	if ($counts['c'] > 0) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("google-merchant")');
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='19' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 20)
{
	$db->query("START TRANSACTION");
	if ($settings['EndiciaShippingLabelsActive'] == 'Yes') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("endicia")');
	if ($settings['StampscomActive'] == 'Yes') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("stamps")');
	if ($settings['AvalaraEnableTax'] == 'Yes') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("avalara")');
	if ($settings['ExactorEnableTax'] == 'Yes') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("exactor")');
	if ($settings['paypalec_Offer_PayPal_BML'] == 'Yes')
	{
		$counts = $db->selectOne('SELECT COUNT(*) as c FROM '.DB_PREFIX.'payment_methods WHERE id = "paypalec" AND active = "Yes"');
		if ($counts['c'] > 0) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("paypal-bml")');
	}
	if ($settings['facebookLogin'] == 'Yes') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("facebook");');
	if ($settings['AffiliateActive'] == 'YES')$db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("idev");');
	if ($settings['mailchimp_enabled'] == '1') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("mailchimp");');
	if ($settings['intuit_anywhere_enabled'] == 'Yes') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("quickbooks");');

	if ($settings['doba_retailer_id'] != '' && $settings['doba_username'] != '' && $settings['doba_password'] != '') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("doba");');

	$counts = $db->selectOne('SELECT COUNT(*) as c FROM '.DB_PREFIX.'products WHERE amazon_id != ""', MYSQL_ASSOC);
	if ($counts['c'] > 0) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("amazon")');

	$counts = $db->selectOne('SELECT COUNT(*) as c FROM '.DB_PREFIX.'products WHERE yahoo_path != ""', MYSQL_ASSOC);
	if ($counts['c'] > 0) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("yahoo")');

	$counts = $db->selectOne('SELECT COUNT(*) as c FROM '.DB_PREFIX.'products WHERE ebay_cat_id != ""', MYSQL_ASSOC);
	if ($counts['c'] > 0) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("ebay")');

	$counts = $db->selectOne('SELECT COUNT(*) as c FROM '.DB_PREFIX.'products WHERE pricegrabber_category != ""', MYSQL_ASSOC);
	if ($counts['c'] > 0) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("pricegrabber")');

	$counts = $db->selectOne('SELECT COUNT(*) as c FROM '.DB_PREFIX.'products WHERE nextag_category != ""', MYSQL_ASSOC);
	if ($counts['c'] > 0) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("nextag")');

	if ($settings['WholesaleCentralUsername'] != '' && $settings['WholesaleCentralPassword'] != '') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("wholesale-central");');
	if ($settings['ShopzillaMerchantId'] != '') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("shopzilla")');

	$bongoCheckoutActive = $db->selectOne('SELECT * FROM '.DB_PREFIX.'payment_methods WHERE id="bongocheckout" AND active="Yes";');
	if ($bongoCheckoutActive) $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("bongo-checkout");');
	if ($settings['bongocheckout_ExtendJS'] != '') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("bongo-extend");');
	if ($settings['RecurringBillingEnabled'] != '0') $db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("recurring");');

	$db->query("UPDATE  ".DB_PREFIX."settings SET value='20' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

//$db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("webgility")');
//$db->query('INSERT INTO '.DB_PREFIX.'enabled_apps (`key`) VALUES("goshopping")');

if ($updateStep < 21)
{
	$db->query("START TRANSACTION");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."gift_cert (id int(11) unsigned NOT NULL auto_increment, oid int(10) unsigned NOT NULL default '0', first_name varchar(255), last_name varchar(255), phone varchar(255), email varchar(255), gift_amount decimal(20,5) unsigned not null, from_name varchar(255), message varchar(255), voucher varchar(20) NOT NULL, email_gift tinyint(1) NOT NULL default '0', balance DECIMAL(20,5) UNSIGNED NOT NULL default 0, ordered_date datetime null, created_date datetime not null, modified_date datetime not null, PRIMARY KEY (id)) ENGINE=MyISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='21' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 22)
{
	$db->query("START TRANSACTION");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."orders_fulfillment (id int(11) unsigned NOT NULL AUTO_INCREMENT, oid int(11) unsigned NOT NULL, status enum('pending', 'completed') NOT NULL default 'pending', tracking_company varchar(128) NULL, tracking_number varchar(255) NULL, created_at datetime NOT NULL, updated_at datetime NOT NULL, primary key(id)) ENGINE=MyISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='22' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 23)
{
	$db->query("START TRANSACTION");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."orders_fulfillment_items (id int(11) unsigned NOT NULL AUTO_INCREMENT, fulfillment_id int(11) unsigned NOT NULL, oid int(11) unsigned NOT NULL, ocid int(11) unsigned NOT NULL, quantity int(11) unsigned NOT NULL, created_at datetime NOT NULL, updated_at datetime NOT NULL, primary key(id)) ENGINE=MyISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci;");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='23' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 24)
{
	$db->query("START TRANSACTION");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('Yes', 'hidden', 3, 'Global Site Settings', 5, 'AdminHttpsUrl', '', '', '', 'url', '', '');");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='24' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 25)
{
	$db->query("START TRANSACTION");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'hidden', 0, 'payment_stripe', 1, 'stripe_ProvisioningData', '', '', '', '', '', '');");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='25' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 26)
{
	$db->query("START TRANSACTION");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'AccountToken', '', '', '', '', '', '');");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='26' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 27)
{
	$db->query("START TRANSACTION");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'QuickStartHidden', '0', '', '', '', '', '');");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='27' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 28)
{
	$db->query("START TRANSACTION");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'QuickStartCompleted', '0', '', '', '', '', '');");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='28' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}


if ($updateStep < 29)
{
	$db->query("START TRANSACTION");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'QuickStartStepsDisplayed', '', '', '', '', '', '');");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='29' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 30)
{
	$db->query("START TRANSACTION");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'QuickStartStepsStatus', '', '', '', '', '', '');");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='30' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 31)
{
	$db->query("START TRANSACTION");
	$db->query("UPDATE ".DB_PREFIX."settings SET value='No' WHERE name='MobileSiteEnabled';");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='31' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 32)
{
	$db->query("START TRANSACTION");
	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.8.0', description='R.2' WHERE name='AppVer';");
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='32' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 33)
{
	//$db->query("START TRANSACTION");
	// Shipping
	$shippingMethods = $db->selectAll('SELECT * FROM '.DB_PREFIX.'shipping_selected WHERE method_id <> "product_level"');

	foreach ($shippingMethods as $shippingMethod)
	{
		$countriesIds = explode(',', $shippingMethod['country']);
		$excludedCountries = explode(',', $shippingMethod['country_exclude']);

		$hasCatchAll = false;
		$usIndex = false;
		$caIndex = false;
		foreach ($countriesIds as $idx => $coid)
		{
			if ($coid == 0)
			{
				$hasCatchAll = true;
			}
			else if ($coid == 1)
			{
				$usIndex = $idx;
			}
			else if ($coid == 2)
			{
				$caIndex = $idx;
			}
		}

		if ($usIndex !== false && count($countriesIds) == 1 || $caIndex !== false && count($countriesIds) == 1)
		{
			//TODO: nothing?
		}
		else
		{
			$methodIsJustUSCA =  $usIndex !== false && $caIndex !== false && count($countriesIds) == 2;
			if ($usIndex !== false && count($countriesIds) > 1) unset($countriesIds[$usIndex]);
			if ($caIndex !== false && count($countriesIds) > 1 && !$methodIsJustUSCA) unset($countriesIds[$caIndex]);

			$duplicateIds = array();
			if (($hasCatchAll || $usIndex !== false) && !in_array(1, $excludedCountries)) $duplicateIds[] = 1;
			if (($hasCatchAll || $caIndex !== false) && !in_array(2, $excludedCountries) && !$methodIsJustUSCA) $duplicateIds[] = 2;

			$initialShippingMethodId = $shippingMethod['ssid'];
			unset($shippingMethod['ssid']);

			foreach ($duplicateIds as $countryId)
			{
				if ($shippingMethod['carrier_id'] == 'custom' ||
					!$db->selectOne('SELECT * FROM '.DB_PREFIX.'shipping_selected WHERE carrier_id="'.$db->escape($shippingMethod['carrier_id']).'" AND method_id="'.$db->escape($shippingMethod['method_id']).'" AND country_exclude <> "'.$countryId.'" AND (country="'.$countryId.'" OR country LIKE "'.$countryId.',%" OR country LIKE "%,'.$countryId.',%" OR country LIKE "%,'.$countryId.'");', MYSQL_ASSOC))
				{
					$db->reset();
					$shippingMethod['country'] = $countryId;
					foreach ($shippingMethod as $key=>$value) $db->assignStr($key, $value);
					$shippingMethodId = $db->insert(DB_PREFIX.'shipping_selected');

					if ($shippingMethod['carrier_id'] == 'custom')
					{
						$rates = $db->selectAll('SELECT * FROM '.DB_PREFIX.'shipping_custom_rates WHERE ssid='.intval($initialShippingMethodId));
						foreach ($rates as $rate)
						{
							$db->reset();
							$db->assignStr('ssid', $shippingMethodId);
							$db->assignStr('rate_min', $rate['rate_min']);
							$db->assignStr('rate_max', $rate['rate_max']);
							$db->assignStr('base', $rate['base']);
							$db->assignStr('price', $rate['price']);
							$db->assignStr('price_type', $rate['price_type']);
							$db->insert(DB_PREFIX.'shipping_custom_rates');
						}
					}
				}
			}

			if (count($countriesIds) > 0)
			{
				$db->reset();
				$db->assignStr('country', implode(',', $countriesIds));
				$db->update(DB_PREFIX.'shipping_selected', 'WHERE ssid = '.intval($initialShippingMethodId));
			}
		}
	}
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='33' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

// Disable global discounts
if ($updateStep < 34)
{
	$db->query("START TRANSACTION");
	$db->query('UPDATE '.DB_PREFIX.'settings SET value="NO" WHERE name="DiscountsActive"');
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='34' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

// Logic for upc/gtin merge, product_upc and product_gtin should populate product_gtin, if product has product_gtin, then leave it, if product does not have product_gtin, but has product_upc then copy product_upc value to product_gtin
if ($updateStep < 35)
{
	$db->query("START TRANSACTION");
	$db->query('UPDATE '.DB_PREFIX.'products SET product_gtin=product_upc WHERE product_gtin=""');
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='35' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

// Inventory_control AttrRuleInc cannot be set now by admin, how do these products get handled in 3.8?
if ($updateStep < 36)
{
	$db->query("START TRANSACTION");
	$db->query('UPDATE '.DB_PREFIX.'products SET inventory_control="Yes" WHERE inventory_control="AttrRuleInc"');
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='36' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

//	// Product page tabs
//	$db->query('UPDATE '.DB_PREFIX.'settings SET value="Flat" WHERE name="CatalogProductPageBottom"');

// Migrate gift_cert data
if ($updateStep < 37)
{
	$db->query("START TRANSACTION");
	$giftCertResult = $db->query("
			SELECT
				o2.oid, o2.first_name, o2.last_name, o2.phone, o2.email, o2.gift_amount,
				o2.from_name, o2.message, o2.voucher, o2.email_gift, o1.balance, o1.date_ordered AS gift_cert_date,
				o.create_date AS order_date
			FROM ".DB_PREFIX."orders_gift_cert AS o2
			LEFT JOIN ".DB_PREFIX."orders AS o ON o.oid = o2.oid
			LEFT JOIN ".DB_PREFIX."ordered_gift_cert AS o1 ON o1.ocid = o2.ocid
		");

	while ($giftCert = $db->moveNext($giftCertResult, MYSQL_ASSOC))
	{
		$db->reset();
		$db->assignStr('oid', $giftCert['oid']);
		$db->assignStr('first_name', $giftCert['first_name']);
		$db->assignStr('last_name', $giftCert['last_name']);
		$db->assignStr('phone', $giftCert['phone']);
		$db->assignStr('email', $giftCert['email']);
		$db->assignStr('gift_amount', $giftCert['gift_amount']);
		$db->assignStr('from_name', $giftCert['from_name']);
		$db->assignStr('message', $giftCert['message']);
		$db->assignStr('voucher', $giftCert['voucher']);
		$db->assignStr('email_gift', $giftCert['email_gift']);
		$db->assignStr('balance', $giftCert['balance']);

		$giftCertDate = $giftCert['gift_cert_date'] == null ? $giftCert['order_date'] : $giftCert['gift_cert_date'];

		$db->assignStr('ordered_date', $giftCertDate);
		$db->assignStr('created_date', $giftCertDate);
		$db->assignStr('modified_date', $giftCertDate);
		$db->insert(DB_PREFIX.'gift_cert');
	}
	$db->query("UPDATE  ".DB_PREFIX."settings SET value='37' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

if ($updateStep < 38)
{
	$db->query("START TRANSACTION");
	$db->query("UPDATE ".DB_PREFIX."orders AS o LEFT JOIN ".DB_PREFIX."orders_content AS oc ON o.oid = oc.oid AND oc.product_type = 'Tangible' SET o.shipping_required = IF(isnull(oc.product_type), 0, 1)");

	$db->query("ALTER TABLE ".DB_PREFIX."admins CHANGE COLUMN rights rights set('all', 'demo', 'categories', 'products', 'users', 'orders', 'emails', 'admins', 'global', 'business', 'content', 'database', 'stats', 'marketing', 'ccs', 'recurring', 'support') NOT NULL default ''");
	$db->query("UPDATE ".DB_PREFIX."settings SET value = 'Yes' WHERE name = 'ShippingInternational'");
	$columns = columnCheck('global_attributes');
	if (!in_array('global_attributes.track_inventory', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."global_attributes ADD COLUMN track_inventory tinyint(1) unsigned not null default '0' AFTER options");
	$columns = columnCheck('products_inventory');
	if (!in_array('products_inventory.product_sku', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products_inventory ADD COLUMN product_sku varchar(64) NOT NULL default '' AFTER is_active");

	$columns = columnCheck('orders_content');
	if (!in_array('orders_content.shipment_id', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN shipment_id int(10) unsigned NOT NULL AFTER oid");
	if (!in_array('orders_content.same_day_delivery', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."orders_content ADD COLUMN same_day_delivery tinyint(1) NOT NULL default '0'");

	$columns = columnCheck('products');
	if (!in_array('products.same_day_delivery', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN same_day_delivery tinyint(1) NOT NULL default '0' AFTER free_shipping");
	if (!in_array('products.updated', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN updated datetime NOT NULL default '0000-00-00 00:00:00' AFTER added");
	if (!in_array('products.zoom_option', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN zoom_option ENUM('global','none','zoom','magnify','magicthumb','imagelayover') NOT NULL DEFAULT 'global' AFTER description");
	if (!in_array('products.search_keywords', $columns))
		$db->query("ALTER TABLE ".DB_PREFIX."products ADD COLUMN search_keywords varchar(1024) not null default '' AFTER recurring_billing_data");

	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."orders_shipments (id int(10) unsigned NOT NULL auto_increment primary key, oid int(10) unsigned NOT NULL, shipping_type varchar(100) NOT NULL default '', shipping_ssid varchar(100) NOT NULL default '', shipping_carrier_id varchar(255) NOT NULL default '', shipping_method_id varchar(100) NOT NULL default '', shipping_cm_name varchar(100) NOT NULL default '', shipping_amount decimal(20, 5) unsigned NOT NULL default '0.00000', remote_order_id int(10) unsigned null, fulfillment_status varchar(128) not null default 'pending', updated_at datetime not null default '0000-00-00 00:00:00') ENGINE=MyISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."shipping_zip_methods (id int(10) unsigned NOT NULL auto_increment, ssid int(10) unsigned NOT NULL, zip_codes text NULL, delivery_days varchar(255) NOT NULL, before_time time NOT NULL default '23:59:59', last_delivery_time time NOT NULL default '00:00:00', delivery_fee decimal(20,5) unsigned NULL, delivery_fee_type enum('amount', 'percentage') NOT NULL default 'amount', position int(10) unsigned NOT NULL default '1', PRIMARY KEY (id)) ENGINE=MyISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci");
	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."oauth_tokens (id int(10) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY, auth_code varchar(256) NULL, token varchar(256) NULL, expires_at datetime NULL, refresh_token varchar(256), client_id varchar(128) NOT NULL, scope varchar(512) NOT NULL)");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_methods (active, url_to_gateway, type, priority, id, class, gateway_name, name, title, enable_ccs, solution_type, countries_include, countries_exclude, message_payment, message_thankyou, description, supports_recurring_billing) VALUES ('No', '', 'cc', 0, 'authorizenet_cim', 'PAYMENT_AUTHORIZENET_CIM', 'Authorize.Net CIM', 'Authorize.Net CIM', 'Credit Card Payment', 0, 'gateway', 'US', '', '<u>Enter Credit Card Information</u>', '<b>Your Order Has Been Sent</b>', '', 1)");
	$authNetCIMID = $db->insertId();
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'Yes', 'VISA', 'Visa', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'Yes', 'MC', 'Master Card', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'Yes', 'AMEX', 'American Express', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'DICL', 'Diners Club', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'JCB', 'JCB', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'CARTE', 'Carte Blanche', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'ABC', 'Australian BankCard', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'DINO', 'Discover/Novus', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'SW', 'Maestro', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'SO', 'Solo', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'ERT', 'enRoute', 'No')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."payment_accepted_cc VALUES (null, ".$authNetCIMID.", 'No', 'UKE', 'Visa Electron', 'No')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_authorizenet_cim', 1, 'authorizenet_cim_Login', '', '', '', '', '', 'API Login ID')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'text', 0, 'payment_authorizenet_cim', 2, 'authorizenet_cim_Transaction_Key', '', '', '', '', '', 'Valid transaction key')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_authorizenet_cim', 1, 'authorizenet_cim_Test_Request', 'FALSE', 'TRUE, FALSE', 'FALSE', '', '', 'Processing mode (TRUE for test mode and FALSE for real processing)')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_authorizenet_cim', 2, 'authorizenet_cim_Auth_Type', 'Auth-Capture', 'Auth-Capture, Auth-Only', 'Auth-Capture', '', '', 'Auth-Capture is the normal transaction method; a transaction is sent to the system for approval, the transaction is approved, the merchant is notified of the approval, and the transaction automatically settles at the end of the business day without any further action by the merchant. Auth-Only stands for Authorization-Only and means obtaining an authorization for a certain amount on a customer\'s credit card without actually charging the card. If the money is not captured within 30 days, the transaction will expire.')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_authorizenet_cim', 3, 'authorizenet_cim_Enable_Cardinal_Centinel', 'No', 'Yes, No', 'No', '', '', '<table height=\"22\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tr><td width=\"20\"><img src=\"images/icons/ic_centinel.gif\"></td><td><a href=\"admin.php?p=payment_cardinal\" style=\"font-size:11px\"><b>Click here to setup Cardinal Centinel Module</b></a></td></tr></table>')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES ('No', 'select', 1, 'payment_authorizenet_cim', 4, 'authorizenet_cim_Duplicate_Window', '120', '0,30,60,120,300', '120', '', '', 'Any transaction submitted with the same card number within specified period of time (in seconds) will be considered a duplicate transaction. Set to 0 to disable.')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No','hidden','0','Catalog Settings','0','ZoomOption','imagelayover','','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No','hidden','0','Catalog Settings','0','ZoomOptionSecondary','imagelayover','','','','','')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'PikflyEnabled', 'No', 'Yes,No', 'No', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'PikflyTimezone', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'text', 0, '', 0, 'PikflyShowAlternativeMethods', 'No', 'Yes,No', 'No', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('Yes', 'select', 3, 'Global Site Settings', 9, 'ForceSslRedirection', 'No', 'Yes,No', 'No', '', '', 'If enabled, every plain HTTP request will be redirected to corresponding secure (SSL) URL. Only works if SSL URL is not the same as website URL')");

	$db->query('
		INSERT INTO '.DB_PREFIX.'orders_shipments (oid,shipping_type,shipping_ssid,shipping_carrier_id,shipping_method_id,shipping_cm_name, fulfillment_status, updated_at)
		SELECT o.oid, o.shipping_ssid, o.shipping_carrier_id, o.shipping_method_id, o.shipping_cm_name, o.shipping_amount, o.fulfillment_status, o.status_date FROM '.DB_PREFIX.'orders o
			LEFT JOIN '.DB_PREFIX.'orders_shipments s ON o.oid=s.oid
		WHERE s.id IS NULL
	');

	$db->query('
		UPDATE '.DB_PREFIX.'orders_content c
		INNER JOIN '.DB_PREFIX.'orders_shipments s ON c.oid = s.oid
		SET c.shipment_id = s.id
	');

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_active','No','Yes,No','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_type','product','product,cart','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_image_type','Product','Product,Thumb,None','','','','')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings VALUES('No','hidden',0,'widgets',0,'widgets_style','1','1,2,3','','','','')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingVIPparcelauthToken', '', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingVIPparcelTestmode', 'no', '', '', '', '', '')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."settings (visible, input_type, group_id, group_name, priority, name, value, options, default_value, validation, caption, description) VALUES ('No', 'hidden', 0, 'Shipping', 0, 'ShippingVIPparcelService', '', '', '', '', '', '')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityMailInternational', 'Priority Mail International', '0,2', 1, 0, 0.01, 70.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityMailExpressInternational', 'Priority Mail Express International', '0,2', 1, 0, 0.01, 70.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'FirstClassPackageInternationalService', 'First-Class Package International Service', '0,2', 1, 0, 0.01, 4.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityMailExpressInternationalFlatRateEnvelope', 'Priority Mail Express International Flat Rate Envelope', '0,2', 1, 0, 0.01, 4.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityMailExpressInternationalFlatRateLegalEnvelope', 'Priority Mail Express International Legal Flat Rate Envelope', '0,2', 1, 0, 0.01, 4.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityMailInternationalSmallFlatRateBox', 'Priority Mail International Small Flat Rate Box', '0,2', 1, 0, 0.01, 4.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityMailInternationalMediumFlatRateBox', 'Priority Mail International Medium Flat Rate Box', '0,2', 1, 0, 0.01, 20.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityMailInternationalLargeFlatRateBox', 'Priority Mail International Large Flat Rate Box', '0,2', 1, 0, 0.01, 20.00, 'lbs')");

	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'Priority', 'Priority Mail', '1', 0, 0, 0.01, 70.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'PriorityExpress', 'Priority Mail Express', '1', 0, 0, 0.01, 70.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'SmallFlatRateBox', 'Priority Mail Small Flat Rate Box', '1', 0, 0, 0.01, 70.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'MediumFlatRateBox', 'Priority Mail Medium Flat Rate Box', '1', 0, 0, 0.01, 70.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'LargeFlatRateBox', 'Priority Mail Large Flat Rate Box ', '1', 0, 0, 0.01, 70.00, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'First', 'First-Class Mail', '1', 0, 0, 0.01, 0.8125, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'StandardPost', 'Standard Mail', '1', 0, 0, 0.01, 1, 'lbs')");
	$db->query("INSERT IGNORE INTO ".DB_PREFIX."shipping_default (carrier_id, carrier_name, method_id, method_name, country, country_exclude, state, weight_min, weight_max, units_weight) VALUES ('vparcel', 'VIPparcel', 'FlatRatePaddedEnvelope', 'Priority Mail Flat Rate Padded Envelope', '1', 0, 0, 0.01, 70, 'lbs')");

	$db->query("CREATE TABLE IF NOT EXISTS ".DB_PREFIX."vparcel_shippinglabels (slid INT(11) NOT NULL AUTO_INCREMENT, tracking_number VARCHAR(50) NOT NULL, pic_number VARCHAR(50) NOT NULL, label_amount DECIMAL(20,5) NOT NULL, label_url VARCHAR(99) NOT NULL, order_id VARCHAR(45) NOT NULL, date_created DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00', PRIMARY KEY (slid)) ENGINE=MyISAM DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci");

	$db->query("UPDATE  ".DB_PREFIX."settings SET value='38' WHERE name='UpdateStep'");
	$db->query("COMMIT");
}

	$db->query("UPDATE ".DB_PREFIX."settings SET value='3.8.2', description='R.1' WHERE name='AppVer'");

	backupChangedFiles();

	$messages[] = array("icon"=>"ic_hint.gif", "text"=>"Successfully backed up changed skin files", "url"=>"", "visible"=>true);

	if (updateLanguages())
	{
		$messages[] = array("icon"=>"ic_hint.gif", "text"=>"Successfully updated language file", "url"=>"", "visible"=>true);
	}
	else
	{
		$messages[] = array("icon"=>"ic_error.gif", "text"=>"Could not update language file. You can click here to update your language file.", "url"=>"admin.php?p=localization", "visible"=>"true");
	}

	recompileSkin();

	global $label_app_name;
	if ((md5(str_replace(' ', '', strtolower($label_app_name))) != '944c32e5f7d6ab022c1101c963ac1370'))
	{
		@unlink('images/admin/logo.png');
	}

	$messages[] = array("icon"=>"ic_hint.gif", "text"=>"Cleared skin cache", "url"=>"", "visible"=>true);

	function backupChangedFiles()
	{
		global $settings, $backup_suffix;

		$skinPath = $settings["GlobalServerPath"]."/content/skins/".escapeFileName($settings["DesignSkinName"])."/";
		$customPath = $settings["GlobalServerPath"]."/content/skins/_custom/skin/";

		$pinkList = array(
			"ModernVista", 'PinnacleBlue', 'BlueSteel', 'Blue3d',
			"NewOrange", "SimpleGreen", "Template3700",
			"Template3701", "Template3702", "Template3703",
			"Template3704", "Template3705", "Template3706",
			"Template3707", "Template3708", "Template3709",
		);

		$fixSkin = !in_array($settings["DesignSkinName"], $pinkList);

		// Place skin files here
		$yellowList = array(
			'javascript/base.js',
			'javascript/common.js',
			'javascript/menu.js',
			'javascript/opc.js',
			'javascript/opc/billing.js',
			'javascript/opc/payment.js',
			'javascript/opc/shipping.js',
			'javascript/products.js',
			'javascript/validators.js',
			'styles/base.css',
			'styles/elements/elements-map.php',
			'templates/admin/design-mode.html',
			'templates/admin/elements/design-dashboard.html',
			'templates/admin/elements/design-dialog-layout-editor.html',
			'templates/admin/elements/design-dialog-new-element.html',
			'templates/admin/elements/design-dialog-styles-editor.html',
			'templates/emails/admin_inactive_accounts_html.html',
			'templates/emails/admin_password_change_reminder_html.html',
			'templates/emails/admin_password_changed_html.html',
			'templates/emails/admin_password_expired_html.html',
			'templates/emails/admin_product_review_html.html',
			'templates/emails/gift_cert_html.html',
			'templates/emails/order_completed_html.html',
			'templates/emails/order_received_html.html',
			'templates/emails/order_shipped_html.html',
			'templates/emails/outofstock_admin_html.html',
			'templates/emails/signup_admin_html.html',
			'templates/layouts/zones/footer.html',
			'templates/layouts/zones/includes/pikfly.html',
			'templates/pages/account/order.html',
			'templates/pages/account/profile.html',
			'templates/pages/account/signup.html',
			'templates/pages/catalog/catalog.html',
			'templates/pages/catalog/elements/element-product-price.html',
			'templates/pages/catalog/elements/element-product-sale-price.html',
			'templates/pages/catalog/elements/element-search-box.html',
			'templates/pages/catalog/views/display.html',
			'templates/pages/catalog/views/displaybox.html',
			'templates/pages/catalog/views/thumb1.html',
			'templates/pages/catalog/views/thumb1clean.html',
			'templates/pages/catalog/views/thumb2.html',
			'templates/pages/catalog/views/thumb3.html',
			'templates/pages/catalog/views/thumb3clean.html',
			'templates/pages/checkout/cart.html',
			'templates/pages/checkout/elements/element-cart-table.html',
			'templates/pages/checkout/opc.html',
			'templates/pages/checkout/opc/opc-billing.html',
			'templates/pages/checkout/opc/opc-invoice.html',
			'templates/pages/checkout/opc/opc-payment.html',
			'templates/pages/checkout/opc/opc-promo-code.html',
			'templates/pages/checkout/shipping-quote.html',
			'templates/pages/gift-certificate/gift-certificate.html',
			'templates/pages/newsletters/unsubscribe.html',
			'templates/pages/product/elements/attributes_js.html',
			'templates/pages/product/elements/element-image.html',
			'templates/pages/product/elements/element-pikfly.html',
			'templates/pages/product/elements/element-secondary-images.html',
			'templates/pages/product/elements/head.html',
			'templates/pages/product/product.html',
			'templates/pages/product/views/style1.html',
			'templates/pages/product/views/style2.html',
			'templates/pages/product/views/style3.html',
			'templates/pages/product/views/style4.html',
			'templates/pages/product/views/style5.html',
			'templates/pages/site/pikfly.html',
			'templates/panels/panel-catalog-bestsellers.html',
			'templates/panels/panel-newsletters-subscribe.html',
			'templates/wrappers/default.html',
			'templates/wrappers/elements/element-default-head.html',
			'LaThreads/fixtures/test_products.sql',
			'LaThreads/javascript/skin.js',
			'LaThreads/skin.xml',
			'LaThreads/styles/elements/elements-map.php',
			'LaThreads/styles/ie7.css',
			'LaThreads/styles/skin.css',
			'LaThreads/styles/theme-default.css',
			'LaThreads/templates/layouts/default.html',
			'LaThreads/templates/layouts/zones/header.html',
			'LaThreads/templates/pages/catalog/catalog.html',
			'LaThreads/templates/pages/catalog/views/display.html',
			'LaThreads/templates/pages/product/elements/element-image.html',
			'LaThreads/templates/pages/product/elements/element-recommended.html',
			'LaThreads/templates/pages/product/elements/element-secondary-images.html',
			'LaThreads/templates/pages/product/elements/element-siblings.html',
			'LaThreads/templates/pages/product/product.html',
			'LaThreads/templates/pages/site/home.html',
			'LaThreads/templates/panels/panel-newsletters-subscribe.html',
			'LaThreads/templates/wrappers/elements/element-custom-head.html',
		);

		foreach ($yellowList as $file)
		{
			if ($fixSkin && is_file($skinPath.$file)) rename($skinPath.$file, $skinPath.$file.".".$backup_suffix);
			if (is_file($customPath.$file)) rename($customPath.$file, $customPath.$file.".".$backup_suffix);
		}
	}

	function columnCheck($tablename)
	{
		global $db;
		static $columns = array();

		$result = $db->selectAll("DESC ".DB_PREFIX.$tablename);
		foreach ($result as $row)
		{
			$columns[] = "$tablename.".$row['Field'];
		}

		return $columns;
	}

	function updateLanguages()
	{
		global $db;
		$languageObject = new Language($db);

		foreach ($languageObject->languages as $language)
		{
			$ret = $languageObject->completeLanguage($language['code']);
			if (!$ret)
				return false;
		}

		return true;
	}

	function updateSettings($key, $value = null)
	{
		global $settings;

		if (!is_array($key))
		{
			if (is_null($value)) throw new Exception('You must pass a value for the settings key');
			$key = array($key => $value);
		}

		foreach ($key as $index => $setting_value)
		{
			if (!isset($settings[$index]))
			{
				$settings[$index] = $setting_value;
			}
		}
	}

	function recompileSkin()
	{
		global $db, $settings;

		$dm = new Ddm_Design_Mode($settings);
		$dm->clearSkinCache();

		view()->assemble('content'.DS.'cache'.DS.'skins'.DS.'_mobile'.DS);
	}
